from typing import Any, Callable, Dict, List, Sequence, Tuple
from pathlib import Path
from xdxsmi.api.node import Node
from xdxsmi.api.xdxgpu import XdxGpu
from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
import time, re

from xdxsmi.api.pyxdxml import *
from enum import Enum, auto

#XDX_VENDOR_ID = 0x10DE
# 1B36 is vendor id for qxl virtual gpu in KVM testing environment
#XDX_VENDOR_ID = 0x1B36

XDX_VENDOR_ID = 0x1EED


#Read version of ddk, umd and kmd
def read_versions():
    try:
        fo = open("/sys/kernel/debug/xdxgpu/gpu00/debug_dump")    
        res = re.match( r'[\s\S]*rogueddk (.*)@[\s\S]*UM info: (.*) @.*[\s\S]*KM info: (.*) @.*[\s\S]*', fo.read())
        fo.close()
    except:
        res = ['N/A', 'N/A', 'N/A', 'N/A']
    return res

#Read version of ddk, umd and kmd
def C_read_versions(handle: Any):
    res = ['N/A', 'N/A', 'N/A', 'N/A']
    ver = xdxml_device_get_driver_version(handle)
    res[1] = ver.DDK.decode('utf-8')
    res[2] = ver.KMD.decode('utf-8')
    res[3] = ver.UMD.decode('utf-8')
    return res

class XdxSmi():
    def __init__(self, is_out_of_band = False) -> None:
        #print("zihan_debug:enter XdxSmi")
        self.is_out_of_band = is_out_of_band
        self.is_init = 0
        if is_out_of_band:
            xdxml_out_of_band_init()
        else:
            xdxml_init()
        self.is_init = 1
        #此处为获取xdxgpu个数即handle的c_lib实现（调试中）
        self.device_num = xdxml_device_get_count()
        
        if self.device_num == 0:
            print("No xdxgpu devices found!")
            exit()  
        #print("device_num is " + str(self.device_num))
        self.handle = []
        for i in range(self.device_num):
            self.handle.append(xdxml_device_get_handle_by_index(i))
        if is_out_of_band:
            # print("process out of manager")
            self.xdxgpus = self.get_out_of_band_xdxgpu(self.handle)
        else:
            self.xdxgpus = self.detect_xdxgpus2(self.handle)
            self.vers = C_read_versions(self.handle[0])
            self.driver_ver = xdxml_device_get_driver_release_version(self.handle)

        #self.xdxgpus = self.detect_xdxgpus(self.handle)
        self.timestp = time.asctime(time.localtime(time.time()))
        self.smi_ver = '1.6'
        self.gpuNums = len(self.xdxgpus)   
        #check if we can query vpu
        self.isVpufunctionValid = False
        if not(is_out_of_band):
            self.isVpufunctionValid = xdxml_get_vpu_functions_valid()          

    #此处为detect_xdxgpus的c_lib调用实现（调试中）
    def detect_xdxgpus2(self, handle) -> List[XdxGpu]:
        xdxgpus = []
        for i in range(self.device_num):
            # xdxgpus.append(XdxGpu(i,handle[i]))
            # get the true card index of device 
            index = xdxml_device_get_card_index_by_handle(handle[i])
            xdxgpus.append(XdxGpu(index, i, handle[i]))
        return xdxgpus

    def get_out_of_band_xdxgpu(self, handle) -> List[XdxGpuInfo]:
        xdxgpus = []
        for i in range(self.device_num):
            # get the true card index of device 
            index = xdxml_device_get_card_index_by_handle(handle[i])
            xdxgpus.append(XdxGpuInfo(index, i, handle[i]))
        return xdxgpus

    #Filter target gpus
    def apply_on_gpus(self, func: Callable[[XdxGpu, Tuple, Dict], List[str]],\
                      idx_list=(), *args, **kwargs)-> List[List[str]]:
        #If idx_list is empty, will apply on all gpus by default
        target_gpus = self.xdxgpus
        if idx_list:
            target_gpus = [gpu for gpu in target_gpus if gpu.drm_index in idx_list]
        return [func(gpu, *args, **kwargs) for gpu in target_gpus]

    #Get target attribute values of GPU
    def get_attributes(nodes: list, attrs: Sequence[str], gpu: XdxGpu):
        #List to record attribute Values
        res = []

        #Attributes belong to clock module
        unitConver1 = ["xdx_acpu_freq_mhz", "xdx_gddr_phys_freq_mhz", \
                       "xdx_gfx_freq_mhz", "xdx_ndf_freq_mhz", \
                       "xdx_host_freq_mhz", "xdx_gfx_freq_mhz", "xdx_cmcu_freq_mhz"]

        #Atrributes belong to power module
        unitConver2 = ["in0_input", "curr1_input", "power1_average", \
            "power1_cap_max", "power1_cap_min", "power1_cap"]

        #Atrributes belong to thermal module
        unitConver3 = ["temp1_input", "temp1_highest", "temp1_crit", "temp1_min", "temp1_max"]

        #Atrributes belong to fan module
        unitConver4 = ["fan1_input", "fan1_target", "pwm1"]

        #Add index for gpu
        res.insert(0, str(gpu.drm_index))

        #Unit conversion
        for node in nodes:
            for attr in attrs:
                if gpu[node][attr] == None:
                    continue
                
                #Unit conversion for attirbutes whose units are MHz
                if (attr in unitConver1):
                    val = int(gpu[node][attr])/1000000000
                    res.append(str(val) + "GHz") \
                        if (val>=1) else res.append(str(val*1000)+"MHz")
                
                #Unit conversion for attributes whose units are mV, mC, mA, uW
                elif (attr in unitConver2):
                    val = int(gpu[node][attr])
                    if attr == "in0_input":
                        res.append(str(val/1000) + "V")
                    elif attr == "curr1_input":
                        res.append(str(val/1000) + "A")
                    else:
                        res.append(str(val/1000) + "W")

                #Unit conversion for attributes whose units are C
                elif (attr in unitConver3):
                    res.append(gpu[node][attr] + "\u00B0" + "C")

                #Add Unit for fan
                elif (attr in unitConver4):
                    res.append(gpu[node][attr]) # + "RPM"
                
                #Attributes which don't need unit conversion, like gpu and name
                else:
                    res.append(gpu[node][attr])
                    
        return res

    #Update the value of the target attribute and return the values of attributes
    @staticmethod
    def generate_attr_writer(nodes: list, attrs: Sequence[str], newVal: Any)\
                             ->Callable[[XdxGpu, Tuple, Dict], List[str]]:
        def inner(gpu: XdxGpu, *args, **kwargs)->str:
            for node in nodes:
                for attr in attrs:
                    if gpu[node][attr] == None:
                        continue
                    gpu[node][attr] = newVal
            return None
        return inner

    #Return the values of attributes
    @staticmethod
    def generate_attr_reader(nodes: list, attrs: Sequence[str])\
                             ->Callable[[XdxGpu, Tuple, Dict], List[str]]:
        def inner(gpu: XdxGpu, *args, **kwargs)->str:
            return XdxSmi.get_attributes(nodes, attrs, gpu)
        return inner
