"""
usage:
   xdxsmi clock [options] get [<ip>...]
   xdxsmi clock [options] set gfx <mhz>

options:
   -h, --help            print this help message
   -f, --filename=file   redirect query output to a specified file
   -l, --loop=sec        set look up interval, whose unit is second
   -i, --index=val       target gpu to read/write attributes, start from 0

ip:
   gfx                   frequency of GPU, whose unit is MHz
"""

from typing import Dict
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *
from xdxsmi.cli.render import Render

clock_map = {
    "acpu": "xdx_acpu_freq",
    "cmcu": "xdx_cmcu_freq",
    "gddr": "xdx_gddr_phys_freq",
    "gfx" : "xdx_gfx_freq",
    "ndf" : "xdx_ndf_freq",
}

def get_gfx_radius(lower_bound: None, upper_bound: None):
    # default optional arguments
    gfx_radius = ["50","100","200","300","400","500","600","700","800","900","1000"]
    if lower_bound == 50:
        if upper_bound == 400:
            gfx_radius = ["50","100","200","300","400"]
        elif upper_bound == 800:
            gfx_radius = ["50","100","200","300","400","500","600","700","800"]
        elif upper_bound == 1000:
            gfx_radius = ["50","100","200","300","400","500","600","700","800","900","1000"]
        elif upper_bound == 1200:
            gfx_radius = ["50","100","200","300","400","500","600","700","800","900","1000","1050","1100","1150","1200"]
        else:
            gfx_radius = ["50","100","200","300","400","500","600","700","800","900","1000"]
    elif lower_bound == 400:
        if upper_bound == 1000:
            gfx_radius = ["400","500","600","700","800","900","1000"]
        elif upper_bound == 1200:
            gfx_radius = ["400","500","600","700","800","900","1000","1050","1100","1150","1200"]
        elif upper_bound == 1300:
            gfx_radius = ["400","500","600","700","800","900","1000","1050","1100","1150","1200", "1250", "1300"]
        else:
            gfx_radius = ["400","500","600","700","800","900","1000"]
    else:
        gfx_radius = ["400","500","600","700","800","900","1000"]
    return gfx_radius

def handle(xdx_smi: XdxSmi, args: Dict):
    #breakpoint()
    # TODO: shall display info for all xdxgpu by default
    # need to add xdxgpu index filter later as well
    #target_gpus = [xdxml_device_get_handle_by_index(0)]
    #target_gpus = [xdx_smi.handle[0]]
    
    index = -1
    num_xdxgpus = len(xdx_smi.xdxgpus)
    try:
        if(args['--index'] == None):
            index = -1                    # default situation
        elif(args['--index'] != None and int(args['--index']) >= 0
            and int(args['--index']) < num_xdxgpus):
            index = int(args['--index'])  # assign index
        else:
            raise ValueError("No devices were found")
    except:
        print("No devices were found")
        exit()
    
    target_gpus = xdx_smi.xdxgpus
    if(index >= 0):
        target_gpus = [xdx_smi.xdxgpus[index]]

    #Set value for attribute 'gfx'
    if args["set"] and args["gfx"] and args["<mhz>"]:
        try:
            for gpu in target_gpus:
                xdev = gpu.handle
                
                lower_bound, upper_bound = xdxml_device_id_to_gfx_bound(xdev, gpu.device_id_num)
                gfx_list = get_gfx_radius(lower_bound, upper_bound)
                gfx_string = ",".join(map(str, gfx_list))
                
                if args["<mhz>"] in gfx_list:
                    newVal = str(int(args["<mhz>"])*1000000)
                    xdxml_device_set_gfx_clock(xdev, newVal)
                else:
                    print ("[XDXSMI ERROR]: Invalid input clock frequency", args["<mhz>"], "MHz")
                    print ("[XDXSMI ERROR]: The gfx clock of this card only supports below frequecies whose unit are MHz:")
                    print ("               ", gfx_string)
                    exit()
        except Exception:
            element_ip = args["<mhz>"]
            exit(f"'xdxsmi clock set gfx {element_ip}' is not a xdxsmi clock command."\
                "See 'xdxsmi clock --help'.")

            

    #Get values for target attributes
    elif args["get"]:
        target_ip = ["gfx"]
        # li: filter the same element between target_ip and args["<ip"]
        if args["<ip>"]:
            target_ip = [ip for ip in target_ip if ip in args["<ip>"]]

        # li: judge the input target_ip is legal
        if len(target_ip) == 0:
            element_ip = next(iter(args["<ip>"]))
            exit(f"'xdxsmi clock get {element_ip}' is not a xdxsmi clock command."\
                "See 'xdxsmi clock --help'.")
        
        data = []
        # i = 0
        header = ["index", "gpu", "name"] + target_ip
        data.insert(0, header)
        # instead traverse handle to traverse gpus, cause gpus include handle and user_index
        for gpu in target_gpus:
            xdev = gpu.handle 
            clkInfo = xdxml_device_get_clock(xdev)
            
            row = [
                str(gpu.user_index),
                xdxml_device_get_pci_info(xdev).pci_dbdf.decode("utf8"),
                xdxml_device_get_device_model(xdev).decode('utf-8')]
            for ip in target_ip:
                # TODO: unifiy the xdxml clkinfo unit to Hz or Mhz in uint type
                row.append(str(int(getattr(clkInfo, clock_map[ip])/1e6)) +'MHz' if gpu.is_host else 'N/A')
            data.append(row)
            # i += 1
        Render.render_table(data, args)

    else:
        # no case falls into this situation?
        exit("%r is not a xdxsmi clock command."\
             "See 'xdxsmi clock --help'." % args['<command>'])

if __name__ == "__main__":
    print(docopt(__doc__))