"""
usage:
   xdxsmi power [options] get [<ip>...]
   xdxsmi power [options] set pwr_warn <mw>
   xdxsmi power [options] set volt <mv>

options:
   -h, --help            print this help message
   -f, --filename=file   redirect query output to a specified file
   -l, --loop=sec        set look up interval, whose unit is second
   -i, --index=val       target gpu to read/write attributes, start from 0

ip:
   volt                  voltage of GPU core, whose unite is V  
   current               current of GPU core, whose unit is A
   pwr                   current power of GPU core, whose unit is W
   pwr_total             total power of GPU, whose unit is W
"""

from typing import Dict
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *
from xdxsmi.cli.render import Render

from get_set_attrs import setAttrs, getAttrs

#Dictionary matching abbreviated and real attribute names 
power_map = {
   "gpu":   "xdx_dbdf",
   "name":  "xdx_product_name",
   "volt":  "in0_input",
   "curr":  "curr1_input",
   "pwr":   "power1_average",
   "pwr_max":  "power1_cap_max",
   "pwr_min":  "power1_cap_min",
   "pwr_warn": "power1_cap",
   "pwr_total": "power1_input"
}
power_unit_map = {
   "volt": "V",
   "current": "A",
   "pwr": "W",
   "pwr_total": "W" 
}

def handle(xdx_smi: XdxSmi, args: Dict):
   ##Collect target gpus
   #target_gpus = [xdx_smi.handle[0]]
   index = -1
   num_xdxgpus = len(xdx_smi.xdxgpus)
   try:
      if(args['--index'] == None):
            index = -1                    # default situation
      elif(args['--index'] != None and int(args['--index']) >= 0
            and int(args['--index']) < num_xdxgpus):
            index = int(args['--index'])  # assign index
      else:
            raise ValueError("No devices were found")
   except:
      print("No devices were found")
      exit()
    
   target_gpus = xdx_smi.xdxgpus
   if(index >= 0):
      target_gpus = [xdx_smi.xdxgpus[index]]
   
   #Set values of attributes 'pwr_warn'
   if args["set"] and args["pwr_warn"] and args["<mw>"]:
      try:
         newVal = str(int(args["<mw>"]))
         for gpu in target_gpus:
            xdev = gpu.handle
            chip_model = xdxml_device_get_device_model(xdev).decode('utf-8')
            # vendor_name: XDX, BITLAND, GOODTIMES, OTHERS
            vendor_name = xdxml_device_get_vendor(xdev).decode('utf-8')
            if vendor_name == "XDX" and gpu.support_power_info == 1:
               xdxml_device_set_power_warn(xdev, newVal)
            else:
               warning = [["Warning: index = {}, chip model = {}, vendor = {}, this card does not support power relation command".format(gpu.user_index, chip_model, vendor_name)]]
               Render.render_table(warning, args, upper = False)

      except:
         element_ip = next(iter(args["<mw>"]))
         exit(f"'xdxsmi power set pwr_warn {element_ip}' is not a xdxsmi power command."\
               "See 'xdxsmi power --help'.")

   #Set values of attributes 'volt'
   elif args["set"] and args["volt"] and args["<mv>"]:
      #760 ~ 880 mv
      try:
         for gpu in target_gpus:
            xdev = gpu.handle
            chip_model = xdxml_device_get_device_model(xdev).decode('utf-8')
            # vendor_name: XDX, BITLAND, GOODTIMES, OTHERS
            vendor_name = xdxml_device_get_vendor(xdev).decode('utf-8')

            if vendor_name == "XDX" and gpu.support_power_info == 1:
               newVal_int = int(args["<mv>"])
               if newVal_int < 760 or newVal_int > 880:
                  print("Warning: You can only set voltage between 760 and 880 mv")
                  exit()
               newVal = str(newVal_int)
               xdxml_device_set_power_volt(xdev, newVal)
            else:
               warning = [["Warning: index = {}, chip model = {}, vendor = {}, this card does not support power relation command".format(gpu.user_index, chip_model, vendor_name)]]
               Render.render_table(warning, args, upper = False)

      except Exception:
         element_ip = args["<mv>"]
         exit(f"'xdxsmi power set volt {element_ip}' is not a xdxsmi power command."\
               "See 'xdxsmi power --help'.")

   #Get values of target attributes   
   elif args["get"]:
      target_ip = ["volt", "current", "pwr", "pwr_total"]
      # li: filter the same element between target_ip and args["<ip"]
      if args["<ip>"]:
         target_ip = [ip for ip in target_ip if ip in args["<ip>"]]

      # li: judge the input target_ip is legal
      if len(target_ip) == 0:
         element_ip = next(iter(args["<ip>"]))
         exit(f"'xdxsmi power get {element_ip}' is not a xdxsmi power command."\
               "See 'xdxsmi power --help'.")

      data = []
      # i = 0
      header = ["index", "gpu", "name"] + target_ip
      #print(header)
      data.insert(0, header)
      for gpu in target_gpus:
         xdev = gpu.handle
         chip_model = xdxml_device_get_device_model(xdev).decode('utf-8')
         # vendor_name: XDX, BITLAND, GOODTIMES, OTHERS
         vendor_name = xdxml_device_get_vendor(xdev).decode('utf-8')

         # if current/power/volt is equals with 0, this card does not support soc power
         # we need to hide the soc power and only show the total_power
         is_hide_soc_power = 0
         if vendor_name == "XDX" and gpu.support_power_info == 1:
            powerInfo = xdxml_device_get_power_info(xdev)
            is_hide_soc_power = 0 if powerInfo.volt and powerInfo.current and powerInfo.pwr else 1

            # warning to show the info about pwr_total
            if("pwr_total" in target_ip):
               warning = [["Warning: pwr_total is the simulated total power of the board, which is subject to error and is for reference only."]]
               Render.render_table(warning, args, upper = False)
         else:
            warning = [["Warning: index = {}, chip model = {}, vendor = {}, this card does not support get power info".format(gpu.user_index, chip_model, vendor_name)]]
            Render.render_table(warning, args, upper = False)

         row = [
               str(gpu.user_index),
               xdxml_device_get_pci_info(xdev).pci_dbdf.decode("utf8"),
               chip_model]
         for ip in target_ip:
            if vendor_name == "XDX" and gpu.support_power_info == 1:
               val = round(getattr(powerInfo, ip)/1e3, 2)
               val = 0 if val < 0 or val > 1000 else val

               # hide soc power data if volt/current/pwer is 0
               if ip in ["volt", "current", "pwr"] and is_hide_soc_power:
                  row.append("N/A")
               else:
                  row.append(str(val) + power_unit_map[ip] if gpu.is_host else 'N/A')
            else:
               row.append("N/A")
         data.append(row)
         # i += 1
      Render.render_table(data, args)
    
   #Stop program if the input command is invalid
   else:
      exit("%r is not a xdxsmi power command."\
             "See 'xdxsmi power --help'." % args['<command>'])

if __name__ == "__main__":
   print(docopt(__doc__))