"""
usage:
   xdxsmi query [options]

options:
    -h, --help                    print this help message
    -f, --filename=file           redirect query output to a specified file
    -x, --xml                     generate XML file to store target GPUs info
    -i, --index=val               target gpu to read/write attributes, start from 0
    -d, --display=type            show selected info
    -l, --loop=sec                set look up interval, whose unit is second
"""

from typing import Dict
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.xdxgpu import XdxGpu
from xdxsmi.api.smi import XdxSmi
from get_set_attrs import setAttrs, getAttrs
import time, re

def getgddrCRCAttrs(gpu: XdxGpu):
    if ((gpu.asci_type == "CHIP_PANGU_A0" and gpu.fm_ver < "v2.2.6") or 
        (gpu.asci_type == "CHIP_PANGU_B0" and gpu.fm_ver < "v1.2.7")):
        gddrCRCAttrs = "\n"
        gddrCRCAttrs += f"""\
        GDDR CRC State (total)
            CRC Write Count         : {gpu.crc_write_cnt[0]}
            CRC Read Count          : {gpu.crc_read_cnt[0]}
            CRC Write Retry Count   : {gpu.crc_write_retry_cnt[0]}
            CRC Read Retry Count    : {gpu.crc_read_retry_cnt[0]}\n"""
        return gddrCRCAttrs
    
    gddrCRCHeader ='\n'
    gddrCRCHeader += f"""\
        GDDR CRC State"""
    gddrCRCHeader +='\n'
    
    gddrCRCBody = f""""""
    for i in range(0, int(gpu.gddr_count, 16)):
        gddrCRCBody += f"""\
            PHY-{i}
              CRC Write Count       : {gpu.crc_write_cnt[i]}
              CRC Read Count        : {gpu.crc_read_cnt[i]}
              CRC Write Retry Count : {gpu.crc_write_retry_cnt[i]}
              CRC Read Retry Count  : {gpu.crc_read_retry_cnt[i]}\n"""
    return gddrCRCHeader + gddrCRCBody

def printAttributes(gpu: XdxGpu, args: Dict):
    processInfo = f"""\
        Process
                GUP ID              : {'N/A'}
                PID                 : {'N/A'}
                Process Name        : {'N/A'}
                Memory Usage        : {'N/A'}"""

    gpuAttrs = f"""\
    GPU {gpu.user_index}
        Product Name                : {gpu.name}
        Core                        : {gpu.core}
        PrimaryID                   : {gpu.primary_id}
        Primary Core ID             : {gpu.primary_core_id}
        Display Mode                : {gpu.dsp_mode}
        Accounting mode             : {gpu.acct_mode}
        GPU UUID                    : {gpu.uuid}
        Minor Number                : {gpu.minor_num}
        VBIOS Version               : {gpu.vbios}
        FW Version                  : {gpu.fm_ver}
        GDDR Count                  : {gpu.gddr_count}
        GDDR Speed                  : {gpu.gddr_speed}
        GDDR Size                   : {gpu.gddr_size}"""
    
    gddrCRCAttrs = getgddrCRCAttrs(gpu)
    # placeholder to show the display results
    # gddrCRCAttrs = f"""\
    #     GDDR CRC State
    #         CRC Write Count         : {gpu.crc_write_cnt}
    #         CRC Read Count          : {gpu.crc_read_cnt}
    #         CRC Write Retry Count   : {gpu.crc_write_retry_cnt}
    #         CRC Read Retry Count    : {gpu.crc_read_retry_cnt}"""
    
    otherAttrs = f"""\
        PCIE Memory bar Type        : {gpu.pcie_mmbar_type}
        Asci Type                   : {gpu.asci_type}
        Pcie Event                  
            Reset Assert            : {gpu.perstAssert} 
            Reset DeAssert          : {gpu.perstDeAssert}
            Link Clock Stable       : {gpu.linkClkStable}
            Link Down               : {gpu.linkDown}
            Reset Assert AcpuAck    : {gpu.perstAssertAcpuAck}
            Reset DeAssert AcpuAck  : {gpu.perstDeAssertAcpuAck}
        PCI
            Bus                     : {gpu.bus}
            Device                  : {gpu.dev}
            Domain                  : {gpu.domain}
            Device Id               : {gpu.device_id}
            Bus Id                  : {gpu.pci_bdf}
            Sub System Id           : {gpu.subsystem_device}
            GPU Link Info
                PCIe Generation
                    Max             : {gpu.pcie_gen_max}
                    Current         : {gpu.pcie_gen_cur}
                Link Width
                    Max             : {gpu.width_max}
                    Current         : {gpu.width_cur}
                Throughput per lane : {gpu.throughput}
        PCIE Error
            PCIE Error Corr         : {gpu.pcie_err_corr}
            PCIE Error Fatal        : {gpu.pcie_err_fatal}
            PCIE Error Non Fatal    : {gpu.pcie_err_nonfat}
        Fan Speed[RPM]              : {"N/A" if gpu.fan_spd=="0" else gpu.fan_spd}
        Performance State           : {gpu.perform_state}
        Clock Throttle Reasons      : {gpu.clk_throt_reasons}
        FB Memory Usage(MiB)
            Total                   : {gpu.fb_total}
            Used                    : {gpu.used_GPU_memory}
            Free                    : {gpu.free_GPU_memory}
        HOST Memory Usage(MiB)
            Total                   : {gpu.sys_total}
            Used                    : {gpu.sys_used}
            Free                    : {gpu.sys_free}
        ECC Mode
            Current                 : {gpu.ecc_curr}
            Pending                 : {gpu.ecc_pend}
        ECC Errors                  : {gpu.ecc_error}
        Temprature                  : {gpu.temp}
        Power Reading               : {gpu.pwr_read}
        Clocks
            Graphics                : {gpu.gfx_clk}
            GDDR                    : {gpu.gddr_clk}
        Application Clocks          : {gpu.app_clk}
        Default Application Clocks  : {gpu.def_app_clk}
        Supported Clock             : {gpu.sup_clk}
        GPU Resource Usage
            2D/TDM                  : {gpu.twoDTDMusage}
            GEOM                    : {gpu.GEOMMusage}  
            3D                      : {gpu.threeDMusage}"""      
              
    gpuMemStr="       Process Memory Usage(MiB)"
    gpuMem =[f"""
            PID                     : {getattr(gpu,f"pro{i}PID")}                   
                NAME                : {getattr(gpu,f"pro{i}NAME")}
                LMA                 : {getattr(gpu,f"pro{i}LMA")}
                Total               : {getattr(gpu,f"pro{i}Total")}"""
                    for i in range(1,gpu.pronum+1)]
    gpuMemStr += "".join(gpuMem)
    
    gpuAttrs += gddrCRCAttrs + otherAttrs
    for i in range(1,gpu.pronum+1):
        getattr(gpu,f"pro{i}PID")
    
    if(args['--filename'] != None):
        fo = open(args['--filename'], "a")
        fo.write('\n')
        fo.write(gpuAttrs)
        fo.write('\n')
        fo.close()
    else:
        print(gpuAttrs)
        # print(gpuMemStr)        

def handle(xdx_smi: XdxSmi, args: Dict):
    index = -1
    num_xdxgpus = len(xdx_smi.xdxgpus)
    try:
        if(args['--index'] == None):
            index = -1                    # default situation
        elif(args['--index'] != None and int(args['--index']) >= 0
            and int(args['--index']) < num_xdxgpus):
            index = int(args['--index'])  # assign index
        else:
            raise ValueError("No devices were found")
    except:
        print("No devices were found")
        exit()
    
    target_gpus = xdx_smi.xdxgpus
    if(index >= 0):
        target_gpus = [xdx_smi.xdxgpus[index]]

    timestp = time.asctime(time.localtime(time.time()))
    header = f"""\
==========================XDXSMI LOG==========================
    Timestamp                       : {timestp}
    Driver Version
        Driver                      : {xdx_smi.driver_ver}                  
        DDK                         : {xdx_smi.vers[1]}
        KMD                         : {xdx_smi.vers[2]}
        UMD                         : {xdx_smi.vers[3]}
    Attached GPUs                   : {xdx_smi.gpuNums}"""

    # show running process
    gpu = target_gpus[0]
    # filter out processes whose LMA is 0
    non_zero_lma_process = [i for i in range(1, gpu.pronum+1) if float(getattr(gpu, f"pro{i}LMA")) > 0]

    gpuMemStr="    top-5 Process Memory Usage(MiB)"
    gpuMem =[f"""
        PID                         : {getattr(gpu,f"pro{i}PID")}                   
            NAME                    : {getattr(gpu,f"pro{i}NAME")}
            LMA                     : {getattr(gpu,f"pro{i}LMA")}
            Total                   : {getattr(gpu,f"pro{i}Total")}"""
                    for i in non_zero_lma_process]
    gpuMemStr += "".join(gpuMem)
    gpuMemStr += "\n"


    if(args['--filename'] != None):
        fo = open(args['--filename'], "a")
        fo.write(header)
        fo.close()
    else:
        print(header)
        print(gpuMemStr)

    for xdxgpu in target_gpus:
        printAttributes(xdxgpu, args)
        xdxgpu.update_dynamic_info()


if __name__ == "__main__":
    print(docopt(__doc__))