"""
usage:
   xdxsmi temp [options] get [<ip>...]

options:
   -h, --help            print this help message
   -f, --filename=file   redirect query output to a specified file
   -l, --loop=sec        set look up interval, whose unit is second
   -i, --index=val       target gpu to read/write attributes, start from 0

ip:
   gddr_temp             current temperature of GDDR
   temp                  current temperature of GPU
   temp_highest          the highest recorded temperature of GPU
   temp_min              temperature of GPU to speed fan
   temp_max              temperature of GPU to underclocking
   temp_crit             temperature of GPU to restart or blue screen
"""

#xdxsmi temp [options] set temp_crit <mhz>
from typing import Dict
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *
from xdxsmi.cli.render import Render
from get_set_attrs import setAttrs, getAttrs
import time
import sys

temp_map = {
    "gpu" : "xdx_dbdf",
    "name": "xdx_product_name",
    "temp": "temp1_input",
    "temp_highest": "temp1_highest",
    "temp_min": "temp1_min",
    "temp_max": "temp1_max",
    "temp_crit": "temp1_crit",
    "gddr_temp": "xdx_gddr_temp"
}
temp_unit_map = {
    "temp": "°C",
    "temp_highest": "°C",
    "temp_min": "°C",
    "temp_max": "°C",
    "temp_crit": "°C",
    "gddr_temp": "°C"
}

def handle(xdx_smi: XdxSmi, args: Dict):
    #Collect target gpus
    #target_gpus = [xdx_smi.handle[0]]
    index = -1
    num_xdxgpus = len(xdx_smi.xdxgpus)
    try:
        if(args['--index'] == None):
            index = -1                    # default situation
        elif(args['--index'] != None and int(args['--index']) >= 0
            and int(args['--index']) < num_xdxgpus):
            index = int(args['--index'])  # assign index
        else:
            raise ValueError("No devices were found")
    except:
        print("No devices were found")
        exit()
    
    target_gpus = xdx_smi.xdxgpus
    if(index >= 0):
        target_gpus = [xdx_smi.xdxgpus[index]]
    #Get values of target attributes   
    if args["get"]:
        target_ip = ["gddr_temp", "temp", "temp_highest", "temp_min", "temp_max", "temp_crit"]
        # li: filter the same element between target_ip and args["<ip"]
        if args["<ip>"]:
            target_ip = [ip for ip in target_ip if ip in args["<ip>"]]
        
        # li: judge the input target_ip is legal
        if len(target_ip) == 0:
            element_ip = next(iter(args["<ip>"]))
            exit(f"'xdxsmi temp get {element_ip}' is not a xdxsmi temp command."\
                "See 'xdxsmi temp --help'.")
        
        data = []
        # i = 0
        header = ["index", "gpu", "name"] + target_ip
        data.insert(0, header)
        for gpu in target_gpus:
            xdev = gpu.handle
            tempInfo = xdxml_device_get_temper_info(xdev)
            
            #print(clkInfo)
            row = [
                str(gpu.user_index),
                xdxml_device_get_pci_info(xdev).pci_dbdf.decode("utf8"),
                xdxml_device_get_device_model(xdev).decode('utf-8')]
            # modify row to show content in target_ip
            for ip in target_ip:
                row.append(str(getattr(tempInfo, ip)) + temp_unit_map[ip] if gpu.is_host else 'N/A')
            data.append(row)
            # i += 1
        Render.render_table(data, args)

    else:
        exit("%r is not a xdxsmi power command."\
             "See 'xdxsmi clock --help'." % args['<command>'])

if __name__ == "__main__":
    print(docopt(__doc__))